
#ifndef COMM_COMMON_H
#define COMM_COMMON_H

#include "secnet.h"

/*----- for all comms -----*/

struct comm_notify_entry {
    comm_notify_fn *fn;
    void *state;
    LIST_ENTRY(comm_notify_entry) entry;
};
LIST_HEAD(comm_notify_list, comm_notify_entry) notify;

struct commcommon { /* must be first so that void* is comm_common* */
    closure_t cl;
    struct comm_if ops;
    struct cloc loc;
    struct comm_notify_list notify;
    struct buffer_if *rbuf;
};

void comm_request_notify(void *commst, void *nst, comm_notify_fn *fn);
void comm_release_notify(void *commst, void *nst, comm_notify_fn *fn);

bool_t comm_notify(struct comm_notify_list *notify, struct buffer_if *buf,
		   const struct comm_addr *ca);
  /* Either: returns True, with message delivered and buffer freed.
   * Or: False, if no-one wanted it - buffer still allocd'd.
   * Ie, like comm_notify_fn. */

void comm_apply(struct commcommon *cc);

#define COMM_APPLY(st,cc,prefix,desc,loc)		\
    (st)=safe_malloc(sizeof(*(st)), desc "_apply");	\
    (cc)->loc=loc;					\
    (cc)->cl.description=desc;				\
    (cc)->ops.sendmsg=prefix##sendmsg;			\
    (cc)->ops.addr_to_string=prefix##addr_to_string;	\
    comm_apply((cc))
   /* Expects in scope: prefix##sendmsg, prefix##addr_to_string. */

#define COMM_APPLY_STANDARD(st,cc,desc,args)				\
    item_t *item=list_elem(args,0);					\
    if (!item || item->type!=t_dict) {					\
	cfgfatal((cc)->loc,desc,"first argument must be a dictionary\n"); \
    }									\
    dict_t *d=item->data.dict;						\
    (cc)->rbuf=find_cl_if(d,"buffer",CL_BUFFER,True,desc,(cc)->loc)

/*----- for udp-based comms -----*/

#define UDP_MAX_SOCKETS 3 /* 2 ought to do really */

#define MAX_AF (AF_INET6>AF_INET?AF_INET6:AF_INET)

struct udpsock {
    union iaddr addr;
    int fd;
    bool_t experienced[/*0=recv,1=send*/2][MAX_AF+1][/*success?*/2];
};

struct udpsocks {
    int n_socks;
    struct udpsock socks[UDP_MAX_SOCKETS];
    /* private for udp_socks_* */
    struct udpcommon *uc; /* link to parent, for cfg, notify list, etc. */
    struct poll_interest *interest;
};

struct udpcommon {
    struct commcommon cc;
    int port;
    string_t authbind;
    bool_t use_proxy;
    union iaddr proxy;
};

bool_t udp_make_socket(struct udpcommon *uc, struct udpsock *us,
		       int failmsgclass);
  /* Fills in us->fd.  Logs any errors with lg_[v]perror. */

void udp_destroy_socket(struct udpcommon *uc, struct udpsock *us);
  /* Idempotent.  No errors are possible. */

const char *af_name(int af);
void udp_sock_experienced(struct log_if *lg, struct udpcommon *uc,
			  const char *socksdesc, struct udpsock *us,
			  bool_t recvsend, int af /* 0 means any */,
			  int r, int errnoval);

void udp_socks_register(struct udpcommon *uc, struct udpsocks *socks);
void udp_socks_deregister(struct udpcommon *uc, struct udpsocks *socks);

#define UDP_APPLY_STANDARD(st,uc,desc)					\
    (uc)->use_proxy=False;						\
    (uc)->authbind=NULL;						\
    (uc)->port=dict_read_number(d,"port",True,"udp",(uc)->cc.loc,0)
    /* Expects in scope:  dict_t *d=...; */

#endif /*COMM_COMMON_H*/
